@@ -1,5 +1,6 @@
 # robot imports
 
+from generic_camunda_client.configuration import Configuration
 from robot.api.deco import library, keyword
 from robot.api.logger import librarylogger as logger
 
@@ -29,7 +30,7 @@ import generic_camunda_client as openapi_client
 from .CamundaResources import CamundaResources
 
 
-@library(scope='GLOBAL')
+@library(scope='SUITE')
 class CamundaLibrary:
     """
     Library for Camunda integration in Robot Framework
@@ -93,12 +94,36 @@ class CamundaLibrary:
     FETCH_RESPONSE: LockedExternalTaskDto = {}
     DEFAULT_LOCK_DURATION = None
 
-    def __init__(self, camunda_engine_url: str = 'http://localhost:8080'):
+    def __init__(self,  host="http://localhost:8080"):
         self._shared_resources = CamundaResources()
-        if camunda_engine_url:
-            self.set_camunda_url(camunda_engine_url)
+        self.set_camunda_configuration(configuration={'host': host})
         self.DEFAULT_LOCK_DURATION = self.reset_task_lock_duration()
 
+    @keyword
+    def set_camunda_configuration(self,configuration: dict):
+        if 'host' not in configuration.keys():
+            raise ValueError(f"Incomplete configuration. Configuration must include at least the Camunda host url:\t{configuration}")
+
+        # weird things happen when dictionary is not copied and keyword is called repeatedly. Somehow robot or python remember the configuration from the previous call
+        camunda_config = configuration.copy()
+
+        host = configuration['host']
+        camunda_config['host'] = url_normalize(f'{host}/engine-rest')
+
+        if 'api_key' in configuration.keys():
+            api_key = configuration['api_key']
+            camunda_config['api_key'] = {
+                'default': api_key
+            }
+            if 'api_key_prefix' in configuration.keys():
+                api_key_prefix = configuration['api_key_prefix']
+                camunda_config['api_key_prefix'] = {
+                'default': api_key_prefix
+            }
+
+        logger.debug(f"New configuration for Camunda client:\t{camunda_config}")
+        self._shared_resources.client_configuration = Configuration(**camunda_config)
+
     @keyword("Set Camunda URL")
     def set_camunda_url(self, url: str):
         """
@@ -153,27 +178,18 @@ class CamundaLibrary:
             try:
                 response: CountResultDto = api_instance.get_external_tasks_count(topic_name=topic, **kwargs)
             except ApiException as e:
-                logger.error(f'Failed to count workload for topic "{topic}":\n{e}')
-                raise e
+                raise ApiException(f'Failed to count workload for topic "{topic}":\n{e}')
 
         logger.info(f'Amount of workloads for "{topic}":\t{response.count}')
         return response.count
 
-    @keyword(name='Deploy Model From File', tags=['deployment'])
-    def deploy_model_from_file(self, path_to_model):
-        """*DEPRECATED*
-
-        Use `Deploy`
-        """
-        logger.warn('Keyword "Deploy Model From File" is deprecated. Use "Deploy" instead.')
-        return self.deploy(path_to_model)
 
     @keyword(name='Deploy', tags=['deployment'])
     def deploy(self, *args):
         """Creates a deployment from all given files and uploads them to camunda.
 
         Return response from camunda rest api as dictionary.
-        Further documentation: https://docs.camunda.org/manual/7.14/reference/rest/deployment/post-deployment/
+        Further documentation: https://docs.camunda.org/manual/latest/reference/rest/deployment/post-deployment/
 
         By default, this keyword only deploys changed models and filters duplicates. Deployment name is the filename of
         the first file.
@@ -202,8 +218,7 @@ class CamundaLibrary:
                                                                                         data=data)
                 logger.info(f'Response from camunda:\t{response}')
             except ApiException as e:
-                logger.error(f'Failed to upload {filename}:\n{e}')
-                raise e
+                raise ApiException(f'Failed to upload {filename}:\n{e}')
 
         return response.to_dict()
 
@@ -225,17 +240,19 @@ class CamundaLibrary:
             fields=fields
         )
 
+        headers=self._shared_resources.api_client.default_headers.copy()
+        headers['Content-Type'] =multipart_data.content_type
+
         logger.debug(multipart_data.fields)
 
         response = requests.post(f'{self._shared_resources.camunda_url}/deployment/create', data=multipart_data,
-                                 headers={'Content-Type': multipart_data.content_type})
+                                 headers=headers)
         json = response.json()
         try:
             response.raise_for_status()
             logger.debug(json)
         except HTTPError as e:
-            logger.error(json)
-            raise e
+            raise ApiException(json)
 
         return json
 
@@ -259,8 +276,7 @@ class CamundaLibrary:
                 response: List[DeploymentDto] = api_instance.get_deployments(**kwargs)
                 logger.info(f'Response from camunda:\t{response}')
             except ApiException as e:
-                logger.error(f'Failed get deployments:\n{e}')
-                raise e
+                raise ApiException(f'Failed get deployments:\n{e}')
 
         return [r.to_dict() for r in response]
 
@@ -286,18 +302,20 @@ class CamundaLibrary:
             serialized_message = api_client.sanitize_for_serialization(correlation_message)
             logger.debug(f'Message:\n{serialized_message}')
 
+            headers=self._shared_resources.api_client.default_headers.copy()
+            headers['Content-Type'] = 'application/json'
+
             try:
                 response = requests.post(f'{self._shared_resources.camunda_url}/message', json=serialized_message,
-                                         headers={'Content-Type': 'application/json'})
+                                         headers=headers)
             except ApiException as e:
-                logger.error(f'Failed to deliver message:\n{e}')
-                raise e
+                raise ApiException(f'Failed to deliver message:\n{e}')
 
         try:
             response.raise_for_status()
         except HTTPError as e:
-            logger.error(response.text)
-            raise e
+            logger.error(e)
+            raise ApiException(response.text)
 
         if correlation_message.result_enabled:
             json = response.json()
@@ -306,14 +324,6 @@ class CamundaLibrary:
         else:
             return {}
 
-    @keyword("Fetch and Lock workloads", tags=['task', 'deprecated'])
-    def fetch_and_lock_workloads(self, topic, **kwargs) -> Dict:
-        """*DEPRECATED*
-
-        Use `fetch workload`
-        """
-        logger.warn('Keyword "Fetch and Lock workloads" is deprecated. Use "Fetch workload" instead.')
-        return self.fetch_workload(topic, **kwargs)
 
     @keyword("Fetch Workload", tags=['task'])
     def fetch_workload(self, topic: str, async_response_timeout=None, use_priority=None, **kwargs) -> Dict:
@@ -322,7 +332,7 @@ class CamundaLibrary:
         Each dictionary representing 1 workload from a process instance.
 
         If a process instance was fetched, the process instance is cached and can be retrieved by keyword
-        `Get recent process instance`
+        `Get Fetch Response`
 
         The only mandatory parameter for this keyword is *topic* which is the name of the topic to fetch workload from.
         More parameters can be added from the Camunda documentation: https://docs.camunda.org/manual/7.14/reference/rest/external-task/fetch/
@@ -332,7 +342,7 @@ class CamundaLibrary:
         Examples:
             | ${input_variables} | *Create Dictionary* | _name=Robot_ |
             | | *start process* | _my_demo_ | _${input_variables}_ |
-            | ${variables} | *fetch and lock workloads* | _first_task_in_demo_ |
+            | ${variables} | *fetch workload* | _first_task_in_demo_ |
             | | *Dictionary Should Contain Key* | _${variables}_ | _name_ |
             | | *Should Be Equal As String* | _Robot_ | _${variables}[name]_ |
 
@@ -363,7 +373,7 @@ class CamundaLibrary:
                 api_response = api_instance.fetch_and_lock(fetch_external_tasks_dto=fetch_external_tasks_dto)
                 logger.info(api_response)
             except ApiException as e:
-                logger.error("Exception when calling ExternalTaskApi->fetch_and_lock: %s\n" % e)
+                raise ApiException("Exception when calling ExternalTaskApi->fetch_and_lock: %s\n" % e)
 
         work_items: List[LockedExternalTaskDto] = api_response
         if work_items:
@@ -379,26 +389,6 @@ class CamundaLibrary:
         variables: Dict[str, VariableValueDto] = self.FETCH_RESPONSE.variables
         return CamundaResources.convert_openapi_variables_to_dict(variables)
 
-    @keyword("Get recent process instance", tags=['task', 'deprecated'])
-    def get_process_instance_id(self):
-        """*DEPRECATED*
-        _Use `get fetch response` instead and check for ``process_instance_id`` element on the fetch reponse:
-        ``fetch_response[process_instance]``_
-
-        Returns cached process instance id from previously called `fetch and lock workloads`.
-
-        *Only this keyword can certainly tell, if workload has been fetched from Camunda*
-
-        Example:
-            | ${variables} | fetch and lock workloads | my_first_task_in_demo | |
-            | Run keyword if | not ${variables} | log | No variables found, but is due to lack of variables or because no workload was available? Must check process instance |
-            | ${process_instance} | get recent process instance | | |
-            | Run keyword if | ${process_instance} | complete task | |
-        """
-        logger.warn('Method is deprecated. Use "Get fetch response"')
-        if self.FETCH_RESPONSE:
-            return self.FETCH_RESPONSE.process_instance_id
-        return self.EMPTY_STRING
 
     @keyword("Get fetch response", tags=['task'])
     def get_fetch_response(self):
@@ -412,7 +402,7 @@ class CamundaLibrary:
             | |
             | *** Tasks *** |
             | | *Create Session* | _alias=camunda_ | _url=http://localhost:8080_ |
-            | | ${variables} | *fetch and lock workloads* | _my_first_task_in_demo_ | |
+            | | ${variables} | *fetch workload* | _my_first_task_in_demo_ | |
             | | ${fetch_response} | *get fetch response* | | |
             | | *POST On Session* | _camunda_ | _engine-rest/external-task/${fetch_response}[id]/complete_ | _json=${{ {'workerId': '${fetch_response}[worker_id]'} }}_ |
         """
@@ -420,8 +410,17 @@ class CamundaLibrary:
             return self.FETCH_RESPONSE.to_dict()
         return self.FETCH_RESPONSE
 
-    @keyword("Drop fetch response", tags=['task'])
+    @keyword("Drop fetch response", tags=['task', 'deprecated'])
     def drop_fetch_response(self):
+        """
+        Removes last process instance from cache.
+
+        *DEPRECATED*
+
+        Actually this keyword does not seem to make sense. Do you have a use case for this? Then contact the project on
+        GitHub, otherwise the keyword will be removed.
+        """
+        logger.warn('DEPRECATED. If you happen to have a use case for this keyword, save the keyword from deletion by contacting the project on GitHub:\thttps://github.com/MarketSquare/robotframework-camunda/issues>')
         self.FETCH_RESPONSE = {}
 
     @keyword("Throw BPMN Error", tags=['task'])
@@ -445,13 +444,17 @@ class CamundaLibrary:
                                                                  external_task_bpmn_error=bpmn_error)
                     self.drop_fetch_response()
                 except ApiException as e:
-                    logger.error(f"Exception when calling ExternalTaskApi->handle_external_task_bpmn_error: {e}\n")
+                    raise ApiException(f"Exception when calling ExternalTaskApi->handle_external_task_bpmn_error: {e}\n")
 
     @keyword("Notify failure", tags=["task"])
     def notify_failure(self, **kwargs):
         """
         Raises a failure to Camunda. When retry counter is less than 1, an incident is created by Camunda.
 
+        You can specify number of retries with the *retries* argument. If current fetched process instance already has
+        *retries* parameter set, the *retries* argument of this keyword is ignored. Instead, the retries counter will
+        be decreased by 1.
+
         CamundaLibrary takes care of providing the worker_id and task_id. *retry_timeout* is equal to *lock_duration* for external tasks.
         Check for camunda client documentation for all parameters of the request body: https://noordsestern.gitlab.io/camunda-client-for-python/7-15-0/docs/ExternalTaskApi.html#handle_failure
 
@@ -466,7 +469,7 @@ class CamundaLibrary:
             with self._shared_resources.api_client as api_client:
                 api_instance = openapi_client.ExternalTaskApi(api_client)
                 if 'retry_timeout' not in kwargs or None is kwargs['retry_timeout'] or not kwargs['retry_timeout']:
-                    kwargs['retry_timeout'] = 60000
+                    kwargs['retry_timeout'] = self.DEFAULT_LOCK_DURATION
 
                 if None is not current_process_instance.retries:
                     kwargs['retries'] = current_process_instance.retries - 1
@@ -478,7 +481,7 @@ class CamundaLibrary:
                                                 external_task_failure_dto=external_task_failure_dto)
                     self.drop_fetch_response()
                 except ApiException as e:
-                    logger.error("Exception when calling ExternalTaskApi->handle_failure: %s\n" % e)
+                    raise ApiException("Exception when calling ExternalTaskApi->handle_failure: %s\n" % e)
 
     @keyword("Get incidents")
     def get_incidents(self, **kwargs):
@@ -497,8 +500,7 @@ class CamundaLibrary:
             try:
                 response: List[IncidentDto] = api_instance.get_incidents(**kwargs)
             except ApiException as e:
-                logger.error(f'Failed to get incidents:\n{e}')
-                raise e
+                raise ApiException(f'Failed to get incidents:\n{e}')
 
         return [incident.to_dict() for incident in response]
 
@@ -509,7 +511,7 @@ class CamundaLibrary:
 
         *Requires `fetch workload` to run before this one, logs warning instead.*
 
-        Additonal variables can be provided as dictionary in _result_set_ .
+        Additional variables can be provided as dictionary in _result_set_ .
         Files can be provided as dictionary of filename and patch.
 
         Examples:
@@ -545,7 +547,7 @@ class CamundaLibrary:
                                                                  complete_external_task_dto=complete_task_dto)
                     self.drop_fetch_response()
                 except ApiException as e:
-                    logger.error(f"Exception when calling ExternalTaskApi->complete_external_task_resource: {e}\n")
+                    raise ApiException(f"Exception when calling ExternalTaskApi->complete_external_task_resource: {e}\n")
 
     @keyword("Download file from variable", tags=['task'])
     def download_file_from_variable(self, variable_name: str) -> str:
@@ -560,7 +562,7 @@ class CamundaLibrary:
                         id=self.FETCH_RESPONSE.process_instance_id, var_name=variable_name)
                     logger.debug(response)
                 except ApiException as e:
-                    logger.error(f"Exception when calling ExternalTaskApi->get_process_instance_variable_binary: {e}\n")
+                    raise ApiException(f"Exception when calling ExternalTaskApi->get_process_instance_variable_binary: {e}\n")
                 return response
 
     @keyword("Unlock", tags=['task'])
@@ -577,7 +579,7 @@ class CamundaLibrary:
                     api_instance.unlock(self.FETCH_RESPONSE.id)
                     self.drop_fetch_response()
                 except ApiException as e:
-                    logger.error(f"Exception when calling ExternalTaskApi->unlock: {e}\n")
+                    raise ApiException(f"Exception when calling ExternalTaskApi->unlock: {e}\n")
 
     @keyword("Start process", tags=['process'])
     def start_process(self, process_key: str, variables: Dict = None, files: Dict = None,
@@ -629,8 +631,7 @@ class CamundaLibrary:
                     start_process_instance_dto=start_process_instance_dto
                 )
             except ApiException as e:
-                logger.error(f'Failed to start process {process_key}:\n{e}')
-                raise e
+                raise ApiException(f'Failed to start process {process_key}:\n{e}')
         logger.info(f'Response:\n{response}')
 
         return response.to_dict()
@@ -646,8 +647,7 @@ class CamundaLibrary:
             try:
                 response = api_instance.delete_process_instance(id=process_instance_id)
             except ApiException as e:
-                logger.error(f'Failed to delete process instance {process_instance_id}:\n{e}')
-                raise e
+                raise ApiException(f'Failed to delete process instance {process_instance_id}:\n{e}')
 
     @keyword("Get all active process instances", tags=['process'])
     def get_all_active_process_instances(self, process_definition_key):
@@ -661,49 +661,50 @@ class CamundaLibrary:
         """
         Queries Camunda for process instances that match certain criteria.
 
-        Be aware, that boolean value must be strings either 'true' or 'false'
+        *Be aware, that boolean value must be strings either 'true' or 'false'*
 
-         async_req: execute request asynchronously
-         sort_by: Sort the results lexicographically by a given criterion. Must be used in conjunction with the sortOrder parameter.
-         sort_order: Sort the results in a given order. Values may be asc for ascending order or desc for descending order. Must be used in conjunction with the sortBy parameter.
-         first_result: Pagination of results. Specifies the index of the first result to return.
-         max_results: Pagination of results. Specifies the maximum number of results to return. Will return less results if there are no more results left.
-         process_instance_ids: Filter by a comma-separated list of process instance ids.
-         business_key: Filter by process instance business key.
-         business_key_like: Filter by process instance business key that the parameter is a substring of.
-         case_instance_id: Filter by case instance id.
-         process_definition_id: Filter by the deployment the id belongs to.
-         process_definition_key: Filter by the key of the process definition the instances run on.
-         process_definition_key_in: Filter by a comma-separated list of process definition keys. A process instance must have one of the given process definition keys.
-         process_definition_key_not_in: Exclude instances by a comma-separated list of process definition keys. A process instance must not have one of the given process definition keys.
-         deployment_id: Filter by the deployment the id belongs to.
-         super_process_instance: Restrict query to all process instances that are sub process instances of the given process instance. Takes a process instance id.
-         sub_process_instance: Restrict query to all process instances that have the given process instance as a sub process instance. Takes a process instance id.
-         super_case_instance: Restrict query to all process instances that are sub process instances of the given case instance. Takes a case instance id.
-         sub_case_instance: Restrict query to all process instances that have the given case instance as a sub case instance. Takes a case instance id.
-         active: Only include active process instances. Value may only be true, as false is the default behavior.
-         suspended: Only include suspended process instances. Value may only be true, as false is the default behavior.
-         with_incident: Filter by presence of incidents. Selects only process instances that have an incident.
-         incident_id: Filter by the incident id.
-         incident_type: Filter by the incident type. See the [User Guide](https://docs.camunda.org/manual/7.15/user-guide/process-engine/incidents/#incident-types) for a list of incident types.
-         incident_message: Filter by the incident message. Exact match.
-         incident_message_like: Filter by the incident message that the parameter is a substring of.
-         tenant_id_in: Filter by a comma-separated list of tenant ids. A process instance must have one of the given tenant ids.
-         without_tenant_id: Only include process instances which belong to no tenant.
-         process_definition_without_tenant_id: Only include process instances which process definition has no tenant id.
-         activity_id_in: Filter by a comma-separated list of activity ids. A process instance must currently wait in a leaf activity with one of the given activity ids.
-         root_process_instances: Restrict the query to all process instances that are top level process instances.
-         leaf_process_instances: Restrict the query to all process instances that are leaf instances. (i.e. don't have any sub instances).
-         variables: Only include process instances that have variables with certain values. Variable filtering expressions are comma-separated and are structured as follows:  A valid parameter value has the form `key_operator_value`. `key` is the variable name, `operator` is the comparison operator to be used and `value` the variable value.  **Note**: Values are always treated as String objects on server side.  Valid `operator` values are: `eq` - equal to; `neq` - not equal to; `gt` - greater than; `gteq` - greater than or equal to; `lt` - lower than; `lteq` - lower than or equal to; `like`. `key` and `value` may not contain underscore or comma characters.
-         variable_names_ignore_case: Match all variable names in this query case-insensitively. If set to true variableName and variablename are treated as equal.
-         variable_values_ignore_case: Match all variable values in this query case-insensitively. If set to true variableValue and variablevalue are treated as equal.
-         _preload_content: if False, the urllib3.HTTPResponse object will
+        == Arguments ==
+        | async_req | execute request asynchronously | 
+        | sort_by | Sort the results lexicographically by a given criterion. Must be used in conjunction with the sortOrder parameter. | 
+        | sort_order | Sort the results in a given order. Values may be asc for ascending order or desc for descending order. Must be used in conjunction with the sortBy parameter. | 
+        | first_result | Pagination of results. Specifies the index of the first result to return. | 
+        | max_results | Pagination of results. Specifies the maximum number of results to return. Will return less results if there are no more results left. | 
+        | process_instance_ids | Filter by a comma-separated list of process instance ids. | 
+        | business_key | Filter by process instance business key. | 
+        | business_key_like | Filter by process instance business key that the parameter is a substring of. | 
+        | case_instance_id | Filter by case instance id. | 
+        | process_definition_id | Filter by the deployment the id belongs to. | 
+        | process_definition_key | Filter by the key of the process definition the instances run on. | 
+        | process_definition_key_in | Filter by a comma-separated list of process definition keys. A process instance must have one of the given process definition keys. | 
+        | process_definition_key_not_in | Exclude instances by a comma-separated list of process definition keys. A process instance must not have one of the given process definition keys. | 
+        | deployment_id | Filter by the deployment the id belongs to. | 
+        | super_process_instance | Restrict query to all process instances that are sub process instances of the given process instance. Takes a process instance id. | 
+        | sub_process_instance | Restrict query to all process instances that have the given process instance as a sub process instance. Takes a process instance id. | 
+        | super_case_instance | Restrict query to all process instances that are sub process instances of the given case instance. Takes a case instance id. | 
+        | sub_case_instance | Restrict query to all process instances that have the given case instance as a sub case instance. Takes a case instance id. | 
+        | active | Only include active process instances. Value may only be true, as false is the default behavior. | 
+        | suspended | Only include suspended process instances. Value may only be true, as false is the default behavior. | 
+        | with_incident | Filter by presence of incidents. Selects only process instances that have an incident. | 
+        | incident_id | Filter by the incident id. | 
+        | incident_type | Filter by the incident type. See the [User Guide](https://docs.camunda.org/manual/latest/user-guide/process-engine/incidents/#incident-types) for a list of incident types. | 
+        | incident_message | Filter by the incident message. Exact match. | 
+        | incident_message_like | Filter by the incident message that the parameter is a substring of. | 
+        | tenant_id_in | Filter by a comma-separated list of tenant ids. A process instance must have one of the given tenant ids. | 
+        | without_tenant_id | Only include process instances which belong to no tenant. | 
+        | process_definition_without_tenant_id | Only include process instances which process definition has no tenant id. | 
+        | activity_id_in | Filter by a comma-separated list of activity ids. A process instance must currently wait in a leaf activity with one of the given activity ids. | 
+        | root_process_instances | Restrict the query to all process instances that are top level process instances. | 
+        | leaf_process_instances | Restrict the query to all process instances that are leaf instances. (i.e. don't have any sub instances). | 
+        | variables | Only include process instances that have variables with certain values. Variable filtering expressions are comma-separated and are structured as follows:  A valid parameter value has the form `key_operator_value`. `key` is the variable name, `operator` is the comparison operator to be used and `value` the variable value.  **Note**: Values are always treated as String objects on server side.  Valid `operator` values are: `eq` - equal to; `neq` - not equal to; `gt` - greater than; `gteq` - greater than or equal to; `lt` - lower than; `lteq` - lower than or equal to; `like`. `key` and `value` may not contain underscore or comma characters. | 
+        | variable_names_ignore_case | Match all variable names in this query case-insensitively. If set to true variableName and variablename are treated as equal. | 
+        | variable_values_ignore_case | Match all variable values in this query case-insensitively. If set to true variableValue and variablevalue are treated as equal. | 
+        | _preload_content | if False, the urllib3.HTTPResponse object will
                                  be returned without reading/decoding response
-                                 data. Default is True.
-         _request_timeout: timeout setting for this request. If one
+                                 data. Default is True. | 
+        | _request_timeout | timeout setting for this request. If one
                                  number provided, it will be total request
                                  timeout. It can also be a pair (tuple) of
-                                 (connection, read) timeouts.
+                                 (connection, read) timeouts. | 
         """
         with self._shared_resources.api_client as api_client:
             api_instance: ProcessInstanceApi = openapi_client.ProcessInstanceApi(api_client)
@@ -711,8 +712,7 @@ class CamundaLibrary:
             try:
                 response: List[ProcessInstanceDto] = api_instance.get_process_instances(**kwargs)
             except ApiException as e:
-                logger.error(f'Failed to get process instances of process:\n{e}')
-                raise e
+                raise ApiException(f'Failed to get process instances of process:\n{e}')
 
         return [process_instance.to_dict() for process_instance in response]
 
@@ -745,8 +745,8 @@ class CamundaLibrary:
             try:
                 response = api_instance.get_process_definitions(**kwargs)
             except ApiException as e:
-                logger.error(f'Failed to get process definitions:\n{e}')
-                raise e
+                raise ApiException(f'Failed to get process definitions:\n{e}')
+
         return response
 
     @keyword("Get Activity Instance", tags=['process'])
@@ -765,12 +765,11 @@ class CamundaLibrary:
             try:
                 response: ActivityInstanceDto = api_instance.get_activity_instance_tree(id)
             except ApiException as e:
-                logger.error(f'failed to get activity tree for process instance with id {id}:\n{e}')
-                raise e
+                raise ApiException(f'failed to get activity tree for process instance with id {id}:\n{e}')
         return response.to_dict()
 
     @keyword("Get Process Instance Variable", tags=['process'])
-    def get_process_instance_variable(self, process_instance_id: str, variable_name: str):
+    def get_process_instance_variable(self, process_instance_id: str, variable_name: str, auto_type_conversion: bool = True):
         """
         Returns the variable with the given name from the process instance with
         the given process_instance_id.
@@ -778,6 +777,7 @@ class CamundaLibrary:
         Parameters:
             - ``process_instance_id``: ID of the target process instance
             - ``variable_name``: name of the variable to read
+            - ``auto_type_conversion``: Converts JSON structures automatically in to python data structures. Default: True. When False, values are not retrieved for JSON variables, but metadata is. Only useful when you want to verify that Camunda holds certain data types.%
 
         == Example ==
         | ${variable} | Get Process Instance Variable |
@@ -785,17 +785,19 @@ class CamundaLibrary:
         | ...         | variable_name=foo |
 
         See also:
-        https://docs.camunda.org/manual/7.5/reference/rest/process-instance/variables/get-single-variable/
+        https://docs.camunda.org/manual/latest/reference/rest/process-instance/variables/get-single-variable/
         """
         with self._shared_resources.api_client as api_client:
             api_instance: ProcessInstanceApi = openapi_client.ProcessInstanceApi(api_client)
 
             try:
                 response = api_instance.get_process_instance_variable(
-                    id=process_instance_id, var_name=variable_name)
+                    id=process_instance_id, var_name=variable_name, deserialize_value=not auto_type_conversion)
             except ApiException as e:
-                logger.error(f'Failed to get variable {variable_name} from '
+                raise ApiException(f'Failed to get variable {variable_name} from '
                              f'process instance {process_instance_id}:\n{e}')
+        if auto_type_conversion:
+            return CamundaResources.convert_variable_dto(response) 
         return response
 
     @keyword("Evaluate Decision", tags=['decision'])
@@ -818,4 +820,4 @@ class CamundaLibrary:
                 return [CamundaResources.convert_openapi_variables_to_dict(r)
                         for r in response]
             except ApiException as e:
-                logger.error(f'Failed to evaluate decision {key}:\n{e}')
+                raise ApiException(f'Failed to evaluate decision {key}:\n{e}')
